@@ -9,6 +9,9 @@ class _TestObservable(object):
         self.center = center
         self.name = name
 
+    def __repr__(self):
+        return "<_TestObservable {name} {id}".format(name=self.name, id=id(self))
+
     def postNotification(self, name):
         self.center.postNotification(name, self)
 
@@ -68,6 +71,30 @@ class NotificationCenterTest(unittest.TestCase):
         center.removeObserver(observer, "A", observable)
         self.assertFalse(center.hasObserver(observer, "A", observable))
 
+    def test_removeObserver_allNotifications(self):
+        center = NotificationCenter()
+        observable1 = _TestObservable(center, "Observable1")
+        observable2 = _TestObservable(center, "Observable2")
+        observer1 = NotificationTestObserver()
+        observer2 = NotificationTestObserver()
+        center.addObserver(observer1, "notificationCallback", "A", observable1)
+        center.addObserver(observer1, "notificationCallback", "B", observable1)
+        center.addObserver(observer2, "notificationCallback", "A", observable1)
+        center.addObserver(observer2, "notificationCallback", "B", observable1)
+        center.addObserver(observer1, "notificationCallback", "A", observable2)
+        center.addObserver(observer1, "notificationCallback", "B", observable2)
+        center.addObserver(observer2, "notificationCallback", "A", observable2)
+        center.addObserver(observer2, "notificationCallback", "B", observable2)
+        center.removeObserver(observer1, "all", observable1)
+        self.assertFalse(center.hasObserver(observer1, "A", observable1))
+        self.assertFalse(center.hasObserver(observer1, "B", observable1))
+        self.assertTrue(center.hasObserver(observer1, "A", observable2))
+        self.assertTrue(center.hasObserver(observer1, "B", observable2))
+        self.assertTrue(center.hasObserver(observer2, "A", observable1))
+        self.assertTrue(center.hasObserver(observer2, "B", observable1))
+        self.assertTrue(center.hasObserver(observer2, "A", observable2))
+        self.assertTrue(center.hasObserver(observer2, "B", observable2))
+
     def test_removeObserver_notification_no_observable(self):
         center = NotificationCenter()
         _TestObservable(center, "Observable")
@@ -81,8 +108,10 @@ class NotificationCenterTest(unittest.TestCase):
         observable = _TestObservable(center, "Observable")
         observer = NotificationTestObserver()
         center.addObserver(observer, "notificationCallback", None, observable)
+        center.addObserver(observer, "notificationCallback", "A", observable)
         center.removeObserver(observer, None, observable)
         self.assertFalse(center.hasObserver(observer, None, observable))
+        self.assertTrue(center.hasObserver(observer, "A", observable))
 
     def test_removeObserver_no_notification_no_observable(self):
         center = NotificationCenter()
@@ -124,7 +153,6 @@ class NotificationCenterTest(unittest.TestCase):
         center.postNotification("B", observable2)
         self.assertEqual(observer.stack[-1], ("A", "Observable2"))
 
-
     def test_postNotification_no_notification_observable(self):
         # no notification, observable
         center = NotificationCenter()
@@ -139,7 +167,6 @@ class NotificationCenterTest(unittest.TestCase):
         self.assertEqual(observer.stack[-1], ("B", "Observable1"))
         center.postNotification("B", observable2)
 
-
     def test_postNotification_no_notification_no_observable(self):
         # no notification, no observable
         center = NotificationCenter()
@@ -165,7 +192,6 @@ class NotificationCenterTest(unittest.TestCase):
         center.addObserver(observer, "notificationCallback", "A", observable1)
         center.addObserver(observer, "notificationCallback", "B", observable1)
         center.addObserver(observer, "notificationCallback", "C", observable2)
-
         center.holdNotifications()
         observable1.postNotification("A")
         self.assertEqual(len(observer.stack), 0)
@@ -190,7 +216,6 @@ class NotificationCenterTest(unittest.TestCase):
         center.addObserver(observer, "notificationCallback", "A", observable1)
         center.addObserver(observer, "notificationCallback", "B", observable1)
         center.addObserver(observer, "notificationCallback", "C", observable2)
-
         center.holdNotifications(observable=observable1)
         observable1.postNotification("A")
         observable1.postNotification("A")
@@ -212,7 +237,6 @@ class NotificationCenterTest(unittest.TestCase):
         center.addObserver(observer, "notificationCallback", "A", observable1)
         center.addObserver(observer, "notificationCallback", "B", observable1)
         center.addObserver(observer, "notificationCallback", "C", observable2)
-
         center.holdNotifications(notification="A")
         observable1.postNotification("A")
         observable1.postNotification("A")
@@ -397,6 +421,128 @@ class NotificationCenterTest(unittest.TestCase):
         center.enableNotifications(observer=observer1)
         self.assertFalse(center.areNotificationsDisabled(observer=observer1))
 
+    def test_addObserver_identifier(self):
+        center = NotificationCenter()
+        observable = _TestObservable(center, "Observable")
+        observer = NotificationTestObserver()
+        center.addObserver(observer, "notificationCallback", "A", observable, identifier="identifier1")
+        center.addObserver(observer, "notificationCallback", "B", observable, identifier="identifier2")
+        expected = [
+            dict(observer=observer, observable=observable, notification="A", identifier="identifier1"),
+            dict(observer=observer, observable=observable, notification="B", identifier="identifier2")
+        ]
+        result = center.findObservations()
+        self.assertEqual(result, expected)
+
+    def test_addObserver_identifierDuplicate(self):
+        center = NotificationCenter()
+        observable = _TestObservable(center, "Observable")
+        observer = NotificationTestObserver()
+        center.addObserver(observer, "notificationCallback", "A", observable, identifier="identifier1")
+        center.addObserver(observer, "notificationCallback", "B", observable, identifier="identifier1")
+        expected = [
+            dict(observer=observer, observable=observable, notification="A", identifier="identifier1"),
+            dict(observer=observer, observable=observable, notification="B", identifier="identifier1")
+        ]
+        self.assertEqual(center.findObservations(), expected)
+
+    def test_removeObserver_identifier(self):
+        center = NotificationCenter()
+        observable = _TestObservable(center, "Observable")
+        observer = NotificationTestObserver()
+        center.addObserver(observer, "notificationCallback", "A", observable, identifier="identifier1")
+        center.addObserver(observer, "notificationCallback", "B", observable, identifier="identifier2")
+        center.removeObserver(observer, "A", observable)
+        center.removeObserver(observer, "B", observable)
+        expected = []
+        self.assertEqual(center.findObservations(), expected)
+
+    def _buildFindObservationsObjects(self):
+        center = NotificationCenter()
+        observable1 = _TestObservable(center, "Observable1")
+        observable2 = _TestObservable(center, "Observable2")
+        observer1 = NotificationTestObserver(name="Observer1")
+        observer2 = NotificationTestObserver(name="Observer2")
+        center.addObserver(observer1, "notificationCallback", "A", observable1, identifier="identifier1-1-A")
+        center.addObserver(observer1, "notificationCallback", "B", observable1, identifier="identifier1-1-B")
+        center.addObserver(observer1, "notificationCallback", "A", observable2, identifier="identifier1-2-A")
+        center.addObserver(observer1, "notificationCallback", "B", observable2, identifier="identifier1-2-B")
+        center.addObserver(observer2, "notificationCallback", "A", observable1, identifier="identifier2-1-A")
+        center.addObserver(observer2, "notificationCallback", "B", observable1, identifier="identifier2-1-B")
+        center.addObserver(observer2, "notificationCallback", "A", observable2, identifier="identifier2-2-A")
+        center.addObserver(observer2, "notificationCallback", "B", observable2, identifier="identifier2-2-B")
+        return center, observer1, observer2, observable1, observable2
+
+    def test_findObservations_observer(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A"),
+            dict(observer=observer1, notification="B", observable=observable1, identifier="identifier1-1-B"),
+            dict(observer=observer1, notification="A", observable=observable2, identifier="identifier1-2-A"),
+            dict(observer=observer1, notification="B", observable=observable2, identifier="identifier1-2-B")
+        ]
+        result = center.findObservations(observer=observer1)
+        self.assertEqual(result, expected)
+
+    def test_findObservations_observable(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A"),
+            dict(observer=observer2, notification="A", observable=observable1, identifier="identifier2-1-A"),
+            dict(observer=observer1, notification="B", observable=observable1, identifier="identifier1-1-B"),
+            dict(observer=observer2, notification="B", observable=observable1, identifier="identifier2-1-B")
+        ]
+        result = center.findObservations(observable=observable1)
+        self.assertEqual(result, expected)
+
+    def test_findObservations_notification(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A"),
+            dict(observer=observer2, notification="A", observable=observable1, identifier="identifier2-1-A"),
+            dict(observer=observer1, notification="A", observable=observable2, identifier="identifier1-2-A"),
+            dict(observer=observer2, notification="A", observable=observable2, identifier="identifier2-2-A")
+        ]
+        result = center.findObservations(notification="A")
+        self.assertEqual(result, expected)
+
+    def test_findObservations_identifier(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A")
+        ]
+        result = center.findObservations(identifier="identifier1-1-A")
+        self.assertEqual(result, expected)
+
+    def test_findObservations_identifierPattern(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A"),
+            dict(observer=observer1, notification="A", observable=observable2, identifier="identifier1-2-A")
+        ]
+        result = center.findObservations(identifier="identifier1-*-A")
+        self.assertEqual(result, expected)
+
+    def test_findObservations_all(self):
+        center, observer1, observer2, observable1, observable2 = self._buildFindObservationsObjects()
+        expected = [
+            dict(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A")
+        ]
+        result = center.findObservations(observer=observer1, notification="A", observable=observable1, identifier="identifier1-1-A")
+        self.assertEqual(result, expected)
+
+    def test_findObservations_noIdentifier(self):
+        center = NotificationCenter()
+        observable = _TestObservable(center, "Observable")
+        observer = NotificationTestObserver()
+        center.addObserver(observer, "notificationCallback", "A", observable, identifier="identifier1")
+        center.addObserver(observer, "notificationCallback", "B", observable)
+        expected = [
+            dict(observer=observer, observable=observable, notification="A", identifier="identifier1"),
+            dict(observer=observer, observable=observable, notification="B", identifier=None)
+        ]
+        result = center.findObservations()
+        self.assertEqual(result, expected)
 
 if __name__ == "__main__":
     unittest.main()
